@@ -5,7 +5,8 @@ import pkgutil
 def get_resource(filename):
     return pkgutil.get_data(__package__, 'data/' + filename)
 
-version = '0.3.5'
+
+version = '0.4.0'
 
 categories = json.loads(get_resource('categories.json').decode())
 sorts = json.loads(get_resource('sorts.json').decode())
@@ -14,5 +15,5 @@ blacklist = set(json.loads(get_resource('blacklist.json').decode()))
 default_headers = {'User-Agent': 'pirate get'}
 default_timeout = 10
 
-default_mirror = 'https://thepiratebay.org/'
+default_mirror = 'https://apibay.org'
 mirror_list = 'https://proxybay.bz/list.txt'
